@@ -15,13 +15,9 @@
 package attest
 
 import (
-	"encoding/base64"
-	"encoding/json"
 	"fmt"
 	"io"
 	"os"
-
-	"github.com/secure-systems-lab/go-securesystemslib/dsse"
 )
 
 func predicateReader(predicatePath string) (io.ReadCloser, error) {
@@ -37,15 +33,3 @@ func predicateReader(predicatePath string) (io.ReadCloser, error) {
 	}
 	return f, nil
 }
-
-func getEnvelopeSigBytes(envelopeBytes []byte) ([]byte, error) {
-	var envelope dsse.Envelope
-	err := json.Unmarshal(envelopeBytes, &envelope)
-	if err != nil {
-		return nil, err
-	}
-	if len(envelope.Signatures) == 0 {
-		return nil, fmt.Errorf("envelope has no signatures")
-	}
-	return base64.StdEncoding.DecodeString(envelope.Signatures[0].Sig)
-}
