@@ -326,6 +326,57 @@ func TestVerifyImageSignatureWithNoChain(t *testing.T) {
 		t.Fatalf("expected verified=true, got verified=false")
 	}
 }
+
+func TestVerifyImageSignatureWithKeyAndCert(t *testing.T) {
+	ctx := context.Background()
+	rootCert, rootKey, _ := test.GenerateRootCa()
+	sv, _, err := signature.NewECDSASignerVerifier(elliptic.P256(), rand.Reader, crypto.SHA256)
+	if err != nil {
+		t.Fatalf("creating signer: %v", err)
+	}
+
+	leafCert, privKey, _ := test.GenerateLeafCert("subject@mail.com", "oidc-issuer", rootCert, rootKey)
+	pemLeaf := pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: leafCert.Raw})
+
+	rootPool := x509.NewCertPool()
+	rootPool.AddCert(rootCert)
+
+	payload := []byte{1, 2, 3, 4}
+	h := sha256.Sum256(payload)
+	sig, _ := privKey.Sign(rand.Reader, h[:], crypto.SHA256)
+
+	// Create a fake bundle
+	pe, _ := proposedEntries(base64.StdEncoding.EncodeToString(sig), payload, pemLeaf)
+	entry, _ := rtypes.UnmarshalEntry(pe[0])
+	leaf, _ := entry.Canonicalize(ctx)
+	rekorBundle := CreateTestBundle(ctx, t, sv, leaf)
+	pemBytes, _ := cryptoutils.MarshalPublicKeyToPEM(sv.Public())
+	rekorPubKeys := NewTrustedTransparencyLogPubKeys()
+	rekorPubKeys.AddTransparencyLogPubKey(pemBytes, tuf.Active)
+
+	opts := []static.Option{static.WithCertChain(pemLeaf, []byte{}), static.WithBundle(rekorBundle)}
+	ociSig, _ := static.NewSignature(payload, base64.StdEncoding.EncodeToString(sig), opts...)
+
+	leafSV, err := signature.LoadECDSASignerVerifier(privKey, crypto.SHA256)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	verified, err := VerifyImageSignature(context.TODO(), ociSig, v1.Hash{},
+		&CheckOpts{
+			SigVerifier:  leafSV,
+			RootCerts:    rootPool,
+			IgnoreSCT:    true,
+			Identities:   []Identity{{Subject: "subject@mail.com", Issuer: "oidc-issuer"}},
+			RekorPubKeys: &rekorPubKeys})
+	if err != nil {
+		t.Fatalf("unexpected error %v", err)
+	}
+	if verified == false {
+		t.Fatalf("expected verified=true, got verified=false")
+	}
+}
+
 func TestVerifyImageSignatureWithInvalidPublicKeyType(t *testing.T) {
 	ctx := context.Background()
 	rootCert, rootKey, _ := test.GenerateRootCa()
@@ -1648,6 +1699,12 @@ func TestVerifyRFC3161Timestamp(t *testing.T) {
 	}
 }
 
+func TestVerifyImageAttestation(t *testing.T) {
+	if _, _, err := VerifyImageAttestation(context.TODO(), nil, v1.Hash{}, nil); err == nil {
+		t.Error("VerifyImageAttestation() should error when given nil attestations")
+	}
+}
+
 // Mock Rekor client
 type mockEntriesClient struct {
 	entries.ClientService
